/*
    Copyright (C) 2005 Ahmed Rezine and Noomene Ben Henda

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information: Ahmed Rezine <Rezine.Ahemd@it.uu.se> and Noomene Ben Henda <Noomene.BenHenda@it.uu.se>
*/

/**@file creator.h
 *ADT for constraint creation.
 *
 *@author Ahmed Rezine and Noomene Ben-Henda
 */

#ifndef CREATOR_H
#define CREATOR_H

#include "process.h"
#include "starpredicate.h"
#include "definition.h"

namespace creator
{
  using namespace starpredicate;
  using namespace process;

  template<class T>
    class Creator
    {
    public:
      Creator() {}
      Creator(const StarPredicate<T>* _p):p(_p) {}
	
    public:
      class iterator
      {
      private:
	typedef typename StarPredicate<T>::constraint_iterator_ref sub_type;
      public:
	typedef T value_type;
      public:
	iterator() {}
	iterator(const Creator<T>* _c,bool _e):c(_c),e(_e) 
	  {
	    T x;def=get_definition(x);ts=T(def);
	    Variable_Table::const_iterator iter=def->begin();
	    
	    while(iter!=def->end())
	      {
		if (iter->second.second)
		  {ts[iter->first]=iter->second.first.second+1; }
		else 
		  {table.push_back(Var_Val(iter->first,iter->second.first.first));
		    ts[iter->first]=iter->second.first.first;}
		iter++;
	      }

	    p_iter=sub_type(&c->get_pred()->begin(ts));

	    while(!e && !c->get_pred()->operator()(ts)){operator++();p_iter=sub_type(&c->get_pred()->begin(ts));}
	  }

	  bool end() const {return e;}
	  
	  iterator& operator=(const iterator& other)
	    {
	      c=other.c;e=other.e;def=other.def;table=other.table;p_iter=other.p_iter;ts=other.ts;
	      return *this;
	    }

	  iterator& operator++()
	    {
	      if (e){return *this;}
	      ++(*p_iter);
	      if (p_iter->end())
		{
		  if(!update(table.size()-1)) {e=true;return *this;}
		  else {p_iter=sub_type(&c->get_pred()->begin(ts));}
		}

	      while(!e && !c->get_pred()->operator()(ts)){return operator++();}
	      return *this;
	    }
	  
	  value_type operator*() const
	  {
	    return p_iter->operator*();
	  }
      private:
	bool update(int j)
	{
	  if (j < 0) {return false;}
	  if (j==0)
	    {
	      if (table[0].second==def->find(table[0].first)->second.first.second)
		{return false;}
	      else 
		{
		  table[0].second++;ts[table[0].first]=table[0].second;
		  return true;
		}
	    }	    
	  else
	    {
	      table[j].second++;
	      if (table[j].second==def->find(table[j].first)->second.first.second+1)
		{
		  table[j].second=def->find(table[j].first)->second.first.first;
		  ts[table[j].first]=table[j].second;
		  return update(j-1);
		}
	      else
		{
		  ts[table[j].first]=table[j].second;
		  return true;
		}
	    }
	}
		    
      protected:
	  const Creator<T>* c;
	  bool e;
	  const Variable_Table* def;
	  std::vector<Var_Val> table;
	  T ts;
	  sub_type p_iter;
      };

    public:

      const StarPredicate<T>* get_pred() const {return p;}
      
      iterator begin() const
	{
	  return iterator(this,false);
	}

    private:
      const StarPredicate<T>* p;	
    };
}

#endif

