/*
    Copyright (C) 2005 Ahmed Rezine and Noomene Ben Henda

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information: Ahmed Rezine <Rezine.Ahemd@it.uu.se> and Noomene Ben Henda <Noomene.BenHenda@it.uu.se>
*/

/**@file minele.h
 *ADT for sequence predicate constraint iterators return type.
 *
 *@author Ahmed Rezine and Noomene Ben-Henda
 */

#ifndef EXCONST_H
#define EXCONST_H

#include "tword.h"
#include "process.h"

/**
 *class extended constraint basically is a word of constraints and a position on that word
 *pointing to the local constraint.
 */

template<class T>
class ExConst
{
  typedef typename TWord<T>::const_iterator iter;
 public:
  ExConst() {}
  ExConst(const TWord<T>& _w):word(_w),curr(word.begin()) {}
  ExConst(const ExConst<T>& other)
    {
      word=TWord<T>(other.word);
      curr=word.begin();
      adjust(other.word.begin(),other.curr,other.word.end());
    }

  ExConst& operator=(const ExConst<T> other)
    {
      word=other.word;
      adjust(other.word.begin(),other.curr,other.word.end());
      return *this;
    }
    
 public:
    TWord<T> word;
    iter curr;
    
 public:
    bool adjust(const iter f,const iter c,const iter l)
    {
      curr=word.begin();iter t=f;
      while(t!=c && t!=l){++t;++curr;}
      return true;
    }
};

using namespace process;

/**
 *class Global extended constraint is an extended constraint plus a Global_Symbol. This is basically the return
 *type for constraint generators in tasks and conditions. It is only needed to keep track of the current position (local constraint).
 */
class GExConst
{
 public:
  GExConst() {}
  GExConst(const process::PWord& _w,const process::Global_Symbol& _g):word(ExConst<process::Process>(_w)),gs(_g){}
    GExConst(const process::PWord& _w,const process::PWord::const_iterator& _iter,const process::Global_Symbol& _g):word(ExConst<process::Process>(_w)),gs(_g)
      {
	process::PWord::const_iterator f=_w.begin(),l=_w.end();
	word.adjust(_w.begin(),_iter,_w.end());
	}
    
      GExConst& operator=(const GExConst& other)
	{
	  word=other.word;
	  gs=other.gs;
	  return *this;
	}

 public:
  ExConst<process::Process> word;
  process::Global_Symbol gs;
};


#endif
