/*
    Copyright (C) 2005 Ahmed Rezine and Noomene Ben Henda

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information: Ahmed Rezine <Rezine.Ahemd@it.uu.se> and Noomene Ben Henda <Noomene.BenHenda@it.uu.se>
*/

/**@file my_functional.h
 *ADT for boolean operator.
 *
 *@author Ahmed Rezine and Noomene Ben-Henda
 */

#ifndef MY_FUNCTIONAL_H
#define MY_FUNCTIONAL_H

#include "printable.h"

namespace my_stl
{
  template<class T1,class T2>
    class My_Binary_Function:public virtual Printable
  {
  public:
    typedef T2 argument_type;
    typedef T1 return_type;

  public:
    virtual T1 operator()(const T2 s1,const T2 s2) const=0;
    virtual ~My_Binary_Function() {}
  };

  template<class T>
    class My_Binary_Op:public My_Binary_Function<bool,T>
    {
  public:
    virtual bool operator()(const T s1,const T s2) const=0;
    virtual My_Binary_Op<T>& negate() const=0;

    virtual ~My_Binary_Op() {}
  };

  class And:public My_Binary_Op<bool>
  {
  public:
    And() {}
    
    bool operator()(const bool s1,const bool s2) const { return s1 && s2; }
    My_Binary_Op<bool>& negate() const;
    void print(std::ostream& out) const {out << "&&";}
  };

  class Or:public My_Binary_Op<bool>
  {
  public:
    Or() {}
    
    bool operator()(const bool s1,const bool s2) const { return s1 || s2; }
    My_Binary_Op<bool>& negate() const;
    void print(std::ostream& out) const {out << "||";}
  };

  class Equal:public My_Binary_Op<int>
  {
  public:
    Equal() {}
    
    bool operator()(const int s1,const int s2) const { return s1 == s2; }
    My_Binary_Op<int>& negate() const;
    void print(std::ostream& out) const {out << "==";}
  };

  class NEqual:public My_Binary_Op<int>
  {
  public:
    NEqual() {}
    
    bool operator()(const int s1,const int s2) const { return s1 != s2; }
    My_Binary_Op<int>& negate() const;
    void print(std::ostream& out) const {out << "!=";}
  };

  class LEqual:public My_Binary_Op<int>
  {
  public:
    LEqual() {}
    
    bool operator()(const int s1,const int s2) const { return s1 <= s2; }
    My_Binary_Op<int>& negate() const;
    void print(std::ostream& out) const {out << "=<";}
  };

  class GEqual:public My_Binary_Op<int>
  {
  public:
    GEqual() {}
    
    bool operator()(const int s1,const int s2) const { return s1 >= s2; }
    My_Binary_Op<int>& negate() const;
    void print(std::ostream& out) const {out << ">=";}
  };

  class Less:public My_Binary_Op<int>
  {
  public:
    Less() {}
    
    bool operator()(const int s1,const int s2) const { return s1 < s2; }
    My_Binary_Op<int>& negate() const;
    void print(std::ostream& out) const {out << "<";}
  };

  class Great:public My_Binary_Op<int>
  {
  public:
    Great() {}
    
    bool operator()(const int s1,const int s2) const { return s1 > s2; } 
    My_Binary_Op<int>& negate() const;
    void print(std::ostream& out) const {out << ">";}
  };

}

#endif
