/*
    Copyright (C) 2005 Ahmed Rezine and Noomene Ben Henda

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information: Ahmed Rezine <Rezine.Ahemd@it.uu.se> and Noomene Ben Henda <Noomene.BenHenda@it.uu.se>
*/

/**@file my_protocol.h
 *This is a possible instance of process data structures. It models a simple
 * semaphore-based mutual exclusion protocol. 
 *
 *@author Ahmed Rezine and Noomene Ben-Henda
 */

#ifndef MYPROTOCOL_H
#define MYPROTOCOL_H
#include "symbol.h"
#include "transition_relation.h"
#include "configuration.h"
#include "tword.h"

#include<iostream>

enum State {Init , Wait , Fina };

/**
 *class process.
 */
class My_Process:public Symbol
{

 public:

  My_Process(): state(Init) {}
  My_Process(State s): state(s) {}

  bool equal(const Symbol& S) const
  {
    const My_Process* P=dynamic_cast<const My_Process*>(&S);
    assert(P);
    return equal(*P);
  }
  
  void print(std::ostream& out) const
  {
    out << state;
  }

 
 public:
  State state;

 public:
  bool equal(const My_Process& P) const
  {
    return state == P.state;
  }

};

/**
 *class global condition.
 */
class My_Global_Condition:public Symbol
{
 public:
  
  My_Global_Condition():semaphore(true) {}
  My_Global_Condition(bool s): semaphore(s) {}

  bool equal(const Symbol& S) const
  { 
    const My_Global_Condition* G=dynamic_cast<const My_Global_Condition*>(&S);
    assert(G);
    return equal(*G);
  }
  
  void print(std::ostream& out) const
  {
    out << semaphore;
  }
  
 public:
  bool semaphore;

 public:
  bool equal(const My_Global_Condition& G) const
  {
    return semaphore == G.semaphore;
  }

};

/**
 *class process word.
 */
typedef TWord<My_Process> My_Word;

typedef Ref<const My_Global_Condition> GCRef;
typedef Ref<const My_Word> WRef;

/**
 *class pre.
 */
class My_Pre:public Pre
{
public:
  My_Pre() {}

  UCSetConf operator()(const Configuration& conf) const
  {
    UCSetConf W;
    GCRef g=conf.global_condition.dyncast<const My_Global_Condition>();
    WRef w=conf.processes.dyncast<const My_Word>();

    My_Word::const_iterator p_beg=w->begin(),p=w->begin(),p_end=w->end();

    while(p!=p_end)
      {
	switch(p->state)
	  {
	  case Init :
	    {
	      if (g->semaphore == true)
		{
		  My_Process pp=My_Process(Fina);
		  My_Global_Condition* pgc=new My_Global_Condition(false);
		  My_Word my_pr;
		  my_pr.push_back(pp);
		  My_Word::iterator pos=my_pr.begin(),pos_end=my_pr.end();
		  my_pr.insert(pos , p_beg , p);++p;
		  my_pr.insert(pos_end , p , p_end);
		  My_Word* pmy_pr=new My_Word(my_pr);
		  Configuration* pconf=new Configuration(pmy_pr,pgc);
		  W.insert(ConfRef(pconf));
		}
	      else{++p;}
	    }
	    break;
	  case Wait :
	    {
	      My_Process pp=My_Process(Init);
	      My_Global_Condition* pgc=new My_Global_Condition(g->semaphore);
	      My_Word my_pr;
	      my_pr.push_back(pp);
	      My_Word::iterator pos=my_pr.begin(),pos_end=my_pr.end();
	      my_pr.insert(pos , p_beg , p);++p;
	      my_pr.insert(pos_end , p , p_end);
	      My_Word* pmy_pr=new My_Word(my_pr);
	      Configuration* pconf=new Configuration(pmy_pr,pgc);
	      W.insert(ConfRef(pconf));
	    }
	    break;
	  case Fina :
	    {
	      if (g->semaphore == false)
		{
		  My_Process pp=My_Process(Wait);
		  My_Global_Condition* pgc=new My_Global_Condition(true);
		  My_Word my_pr;
		  my_pr.push_back(pp);
		  My_Word::iterator pos=my_pr.begin(),pos_end=my_pr.end();
		  my_pr.insert(pos , p_beg , p);++p;
		  my_pr.insert(pos_end , p , p_end);
		  My_Word* pmy_pr=new My_Word(my_pr);
		  Configuration* pconf=new Configuration(pmy_pr,pgc);
		  W.insert(ConfRef(pconf));
		}
	      else{++p;}
	    }
	    break;
	  }
      }
    
    return W;
  }

  UCSetConf operator()(const ConfRef conf) const
  { return operator()(*conf); }

};

#endif

