/*  
    This file is part of yatesParse.

    yatesParse is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    yatesParse is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with yatesParse; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "options_handler.h"

ArgumentHandler::ArgumentHandler(bool tmpGenerateHelp) {
  numberArgs = 0;
  generateHelp = tmpGenerateHelp;
}

void ArgumentHandler::add_option(std::string tmpLongOption, std::string tmpShortOption, int (*tmpCallThis)(std::string argumentValue), std::string tmpOptionDescription, bool tmpCallOnce, bool tmpValueExists) {
  longOption.push_back(tmpLongOption);
  shortOption.push_back(tmpShortOption);
  callThis.push_back(tmpCallThis);
  optionDescription.push_back(tmpOptionDescription);
  callOnce.push_back(tmpCallOnce);
  beenCalled.push_back(0);
  argumentValue.push_back("NULL");
  valueExists.push_back(tmpValueExists);
  numberArgs++;
}

void ArgumentHandler::print() {
  std::cout << programInformation << std::endl;
  std::cout << "Usage: " << usageInformation << std::endl;
  std::cout << "Options: ";
  for (unsigned int iter = 0; iter < numberArgs; ++iter) {
    if (shortOption[iter].compare("NULL") == 0) {
      std::cout << "\n\t" << longOption[iter] <<  "\t\t" << optionDescription[iter];
    } else {
      std::cout << "\n\t" << longOption[iter] << ", " << shortOption[iter] << "\t" << optionDescription[iter];
    }
  }
  std::cout << "\n";
}

void ArgumentHandler::set_usage_information(std::string tmpUsageInformation) {
  usageInformation = tmpUsageInformation;
}

void ArgumentHandler::set_program_information(std::string tmpProgramInformation) {
  programInformation = tmpProgramInformation;
}

void ArgumentHandler::set_generate_help(bool tmpGenerateHelp) {
  generateHelp = tmpGenerateHelp;
}

//not responsible for any headaches that will ensue as a result of this function
void ArgumentHandler::parse(unsigned int argc, char* argv[]) {
  unsigned int iter = 0;
  unsigned int iter2 = 0;
  while (iter < argc) {
    for (iter2=0; iter2 < numberArgs; iter2++) { 
      if (generateHelp == 1 && (!argstrcmp(argv[iter], "--help"))) {
	ArgumentHandler::print();
	exit(0);
      } else {
	if (std::string(argv[iter]) == longOption[iter2] || (std::string(argv[iter]) == shortOption[iter2] && std::string(argv[iter]) != "NULL")) {
	  if (beenCalled[iter2] == 0 || (beenCalled[iter2] == 1 && callOnce[iter2] == 0)) {
	    argumentValueNumber = iter + 1;
	    if (argumentValueNumber >= argc) {
	      argumentValue[iter2] = "NULL";
	    } else {
	      argumentValue[iter2] = argv[argumentValueNumber];
	    }
	    callThis[iter2](argumentValue[iter2]);
	    beenCalled[iter2] = 1;
	  }	// if loop close
	}	// if loop close 
      }		// if loop close
    }		// for loop close
    iter++;
  }		// while loop close
}

int ArgumentHandler::argstrcmp(char *x, char *y)
{
  while (*x && (*x == *y))
    ++x, ++y;
  return (*x - *y);
}

