%{
#include "process.h"
#include "configuration.h"
#include "predicate.h"
#include "variable.h"
#include "starpredicate.h"
#include "starquantifier.h"
//#include "creator.h"
#include "condition.h"
#include "guard.h"
#include "transform.h"
#include "rtransform.h"
#include "my_functional.h"
#include "task.h"
#include "action.h"
#include "transition.h"
#include "starpre.h"

#include <iostream>

using namespace std;
using namespace process;
using namespace predicate;
using namespace variable;
using namespace starpredicate;
using namespace starquantifier;
using namespace guard;
using namespace my_stl;
using namespace transform;
using namespace rtransform;
using namespace action;
//using namespace creator;

extern process::Variable_Table Process_Definition;
extern process::Variable_Table Global_Definition;

//extern Configuration* conf_init;
//extern Configuration* conf_final;
extern Transitions* trans;
extern UCSetConf* Init;
extern UCSetConf* Final;

extern int line_no;
extern int yylex();

void yyerror(const char* s);
int nbr=0;

%}

%union {
	int n;
	bool b;
	std::string* str;
	std::pair<int, int>* range;
	StarFunction<Process>* proc_var;
	StarFunction<Global_Symbol>* glob_var;
	StarPredicate< Process>* pred_loc;
	StarIntOpPredicate< Process>* atomic_loc;
	StarPredicate< Global_Symbol>* pred_glob;
	StarIntOpPredicate< Global_Symbol>* atomic_glob;
	Process* proc_specs;
	Variable_Table* var_tab; 
	PWord* word_specs;
//	Configuration* conf_specs;
	Global_Symbol* glob_specs;
	Condition* cond;
	Guard* guard;
	Action* action;
	Transitions* transitions;
};

%token TOKEN_SEMCOLON;
%token TOKEN_COMMA;
%token TOKEN_COLON;
%token TOKEN_STAR;

%token TOKEN_LOCAL;
%token TOKEN_GLOBAL;
%token TOKEN_TRANSITION;

%token TOKEN_OR; 
%token TOKEN_AND; 
%token TOKEN_EQUAL;
%token TOKEN_DIFFERENT;
%token TOKEN_GREAT;
%token TOKEN_LESS;
%token TOKEN_GEQUAL;
%token TOKEN_LEQUAL;
%token TOKEN_FORALLA;
%token TOKEN_FORALLL;
%token TOKEN_FORALLR;
%token TOKEN_ITEXISTSA;
%token TOKEN_ITEXISTSL;
%token TOKEN_ITEXISTSR;
%token TOKEN_NEW;

%token TOKEN_LEFTBR;
%token TOKEN_RIGHTBR;
%token TOKEN_LEFTP;
%token TOKEN_RIGHTP;
%token TOKEN_DOTDOT;
%token TOKEN_BOOL;
%token TOKEN_IMPLY;
%token TOKEN_ASSIGN;
%token TOKEN_INITIAL;
%token TOKEN_FINAL;

%token TOKEN_TRUE;
%token TOKEN_FALSE;

%token <str>ID 
%token <n>INT

%type <range> range
%type <proc_var> proc_var
%type <glob_var> glob_var
%type <pred_loc> pred_loc 
%type <pred_glob> pred_glob
%type <atomic_loc> atomic_loc 
%type <atomic_glob> atomic_glob
%type <proc_specs> proc_specs
%type <word_specs> word_specs
//%type <conf_specs> conf_specs
%type <glob_specs> glob_specs
%type <n> op
%type <cond> cond
%type <guard> guard
%type <action> action
%type <transitions> transitions
%%
 
program: TOKEN_LOCAL var_loc TOKEN_GLOBAL var_glob TOKEN_TRANSITION transitions TOKEN_INITIAL init_confs TOKEN_FINAL final_confs
{	
	trans=$6;
};

var_loc: 
{ 
	Process_Definition.clear();
}
| var_loc ID range
{
	if(Process_Definition.find(*$2)!= Process_Definition.end())
		yyerror("error: multiple variable definition ");	  
	Process_Definition.define_variable(*$2, $3->first, $3->second, false);
	delete($2);
	delete($3);
}
| var_loc TOKEN_BOOL ID
{
	if(Process_Definition.find(*$3)!= Process_Definition.end())
		yyerror("error: multiple variable definition ");	  
	Process_Definition.define_variable(*$3, 0, 1, false);
	delete($3);
}
| var_loc ID range TOKEN_STAR
{
	if(Process_Definition.find(*$2)!= Process_Definition.end())
		yyerror("error: multiple variable definition ");	  
	Process_Definition.define_variable(*$2, $3->first, $3->second, true);
	delete($2);
	delete($3);	
}
| var_loc TOKEN_BOOL ID TOKEN_STAR
{
	if(Process_Definition.find(*$3)!= Process_Definition.end())
		yyerror("error: multiple variable definition ");	  
	Process_Definition.define_variable(*$3, 0, 1, true);
	delete($3);
};


var_glob:
{ 
	Global_Definition.clear();
}
| var_glob ID range
{
	if(Global_Definition.find(*$2)!= Global_Definition.end())
		yyerror("error: multiple variable definition ");	  
	Global_Definition.define_variable(*$2, $3->first, $3->second, false);
	delete($2);
	delete($3);	
}
| var_glob TOKEN_BOOL ID 
{
	if(Global_Definition.find(*$3)!= Global_Definition.end())
		yyerror("error: multiple variable definition ");	  
	Global_Definition.define_variable(*$3, 0, 1, false);
	delete($3);
}
| var_glob ID range TOKEN_STAR
{
	if(Global_Definition.find(*$2)!= Global_Definition.end())
		yyerror("error: multiple variable definition ");	  
	Global_Definition.define_variable(*$2, $3->first, $3->second, true);
	delete($2);
	delete($3);	
}
| var_glob TOKEN_BOOL ID TOKEN_STAR
{
	if(Global_Definition.find(*$3)!= Global_Definition.end())
		yyerror("error: multiple variable definition ");	  
	Global_Definition.define_variable(*$3, 0, 1, true);
	delete($3);
};

range: TOKEN_LEFTBR INT TOKEN_DOTDOT INT TOKEN_RIGHTBR
{
	if($2 > $4)
	  yyerror("error: invalid range ");
	$$ = new std::pair<int, int>;
	$$->first=$2;
	$$->second=$4;
};

transitions: guard TOKEN_IMPLY action TOKEN_SEMCOLON
{
	$$ = new Transitions;
	Transition* tr = new Transition($1, $3);
//	cout << *tr << endl;
	$$->push_back(tr);
}
| transitions guard TOKEN_IMPLY action TOKEN_SEMCOLON
{
	$$ = $1;
	Transition* tr = new Transition($2, $4);
	//cout << *tr << endl;
	$$->push_back(tr);
};


guard: TOKEN_NEW
{
	$$ = new Guard;
	$$->push_back(new TNew_Condition());
}
| cond
{
	$$ = new Guard;
	$$->push_back($1);
}
| guard TOKEN_AND cond
{
	$$->push_back($3);
};

cond: TOKEN_LOCAL pred_loc
{
	$$=new TLocal_Condition($2);
}
| TOKEN_GLOBAL pred_glob
{
	$$=new TGlobal_Condition($2);
}
| TOKEN_FORALLA pred_loc 
{	
	$$=new TGlobal_Condition_Quantified(new StarForAllA<Process, PWord::const_iterator>($2));
}
| TOKEN_FORALLL pred_loc 
{	
	$$=new TGlobal_Condition_Quantified(new StarForAllL<Process, PWord::const_iterator>($2));
}
| TOKEN_FORALLR pred_loc 
{	
	$$=new TGlobal_Condition_Quantified(new StarForAllR<Process, PWord::const_iterator>($2));
}
| TOKEN_ITEXISTSA pred_loc
{
	$$=new TGlobal_Condition_Quantified(new StarItExistsA<Process, PWord::const_iterator>($2));
}
| TOKEN_ITEXISTSL pred_loc
{	
	$$=new TGlobal_Condition_Quantified(new StarItExistsL<Process, PWord::const_iterator>($2));
}
| TOKEN_ITEXISTSR pred_loc
{	
	$$=new TGlobal_Condition_Quantified(new StarItExistsR<Process, PWord::const_iterator>($2));
}
;


action: ID TOKEN_ASSIGN INT
{
	if((Process_Definition.find(*$1)== Process_Definition.end()) & (Global_Definition.find(*$1)== Global_Definition.end()))
		{ yyerror("error: undefined variable "); }	  

	if(Process_Definition.find(*$1)!= Process_Definition.end() & !Process_Definition.in_range(*$1, $3))
		yyerror("warning: local variable out of range ");	  

	if(Global_Definition.find(*$1)!= Global_Definition.end() & !Global_Definition.in_range(*$1, $3))
		yyerror("warning: global variable out of range ");	  

	$$ = new Action; 
	if(Process_Definition.find(*$1)!= Process_Definition.end())
	{
		$$->push_back(new TLocal_Task(new RAssignment<Process>(*$1, $3)));
	}else {
		$$->push_back(new TGlobal_Task(new RAssignment<Global_Symbol>(*$1, $3)));
		}
	delete($1);	

}
| TOKEN_FORALLA ID TOKEN_ASSIGN INT
{
	if((Process_Definition.find(*$2)== Process_Definition.end()))
		{ yyerror("error: undefined variable "); }	  

	if(Process_Definition.find(*$2)!= Process_Definition.end() & !Process_Definition.in_range(*$2, $4))
		yyerror("warning: local variable out of range ");	  

	$$ = new Action; 
	$$->push_back(new TGlobal_Sequence_Task(new RSequence_Transform_A<Process, PWord::const_iterator>
												(new RAssignment<Process>(*$2, $4))));
	delete($2);	
}
| action TOKEN_AND ID TOKEN_ASSIGN INT
{
	if((Process_Definition.find(*$3)== Process_Definition.end()) & (Global_Definition.find(*$3)== Global_Definition.end()))
		yyerror("error: undefined variable ");	  

	if(Process_Definition.find(*$3)!= Process_Definition.end() & !Process_Definition.in_range(*$3, $5))
		yyerror("warning: local variable out of range ");	  

	if(Global_Definition.find(*$3)!= Global_Definition.end() & !Global_Definition.in_range(*$3, $5))
		yyerror("warning: global variable out of range ");	  

	if(Process_Definition.find(*$3)!= Process_Definition.end())
	$$->push_back(new TLocal_Task(new RAssignment<Process>(*$3, $5)));
	else 
	$$->push_back(new TGlobal_Task(new RAssignment<Global_Symbol>(*$3, $5)));
	delete($3);
}
| action TOKEN_AND TOKEN_FORALLA ID TOKEN_ASSIGN INT
{
	if((Process_Definition.find(*$4)== Process_Definition.end()))
		{ yyerror("error: undefined variable "); }	  

	if(Process_Definition.find(*$4)!= Process_Definition.end() & !Process_Definition.in_range(*$4, $6))
		yyerror("warning: local variable out of range ");	  

	$$->push_back(new TGlobal_Sequence_Task(new RSequence_Transform_A<Process, PWord::const_iterator>
												(new RAssignment<Process>(*$4, $6))));

	delete($4);
}; 

 

pred_loc: atomic_loc
{
	$$ = $1; 
}
|TOKEN_LEFTP pred_loc TOKEN_AND atomic_loc TOKEN_RIGHTP
{
	 $$ = new StarAndPredicate<Process>($2,$4);
}
| TOKEN_LEFTP pred_loc TOKEN_OR atomic_loc TOKEN_RIGHTP
{
	$$ = new StarOrPredicate<Process>($2,$4);	
}
|TOKEN_LEFTP atomic_loc TOKEN_AND pred_loc TOKEN_RIGHTP
{
	 $$ = new StarAndPredicate<Process>($4,$2);
}
| TOKEN_LEFTP atomic_loc TOKEN_OR pred_loc TOKEN_RIGHTP
{
	$$ = new StarOrPredicate<Process>($4,$2);	
}
;

atomic_loc: proc_var op proc_var
{
	switch($2)
	{
	 case 0:
	  $$= new StarIntOpPredicate<Process>($3,$1,new Equal);
	  break;
	 case 1:
	  $$= new StarIntOpPredicate<Process>($3,$1,new NEqual);
	  break;
	 case 2:
	  $$= new StarIntOpPredicate<Process>($1,$3,new LEqual);
	  break;
	 case 3:
	  $$= new StarIntOpPredicate<Process>($1,$3,new GEqual);
	  break;
	 case 4:
	  $$= new StarIntOpPredicate<Process>($1,$3,new Less);
	  break;
	 case 5:
	  $$= new StarIntOpPredicate<Process>($1,$3,new Great);
	  break;
	}
} 
|TOKEN_LEFTP atomic_loc TOKEN_RIGHTP
{
	$$ = $2;
}
;

proc_var: ID
{
	$$=new StarVariable<Process>(*$1);
	delete($1);
}
| INT
{
	$$ = new StarConstant<Process>($1);
};
	
pred_glob: atomic_glob
{
	$$ = $1; 
}
| TOKEN_LEFTP pred_glob TOKEN_AND atomic_glob TOKEN_RIGHTP
{
	 $$ = new StarAndPredicate<Global_Symbol>($2,$4);
}
| TOKEN_LEFTP pred_glob TOKEN_OR atomic_glob TOKEN_RIGHTP
{
	$$ = new StarOrPredicate<Global_Symbol>($2,$4);	
}
;

atomic_glob: glob_var op glob_var
{
	switch($2)
	{
	 case 0:
	  $$= new StarIntOpPredicate<Global_Symbol>($3,$1,new Equal);
	  break;
	 case 1:
	  $$= new StarIntOpPredicate<Global_Symbol>($3,$1,new NEqual);
	  break;
	 case 2:
	  $$= new StarIntOpPredicate<Global_Symbol>($1,$3,new LEqual);
	  break;
	 case 3:
	  $$= new StarIntOpPredicate<Global_Symbol>($1,$3,new GEqual);
	  break;
	 case 4:
	  $$= new StarIntOpPredicate<Global_Symbol>($1,$3,new Less);
	  break;
	 case 5:
	  $$= new StarIntOpPredicate<Global_Symbol>($1,$3,new Great);
	  break;
	}
};

glob_var: ID
{
	$$=new StarVariable<Global_Symbol>(*$1);
	delete($1);
}
| INT
{
	$$ = new StarConstant<Global_Symbol>($1);
};


op: TOKEN_EQUAL    {$$ = 0;}
    | 
    TOKEN_DIFFERENT{$$ = 1;}
    | 
    TOKEN_LEQUAL   {$$ = 2;}
    | 
    TOKEN_GEQUAL   {$$ = 3;}
    | 
    TOKEN_LESS     {$$ = 4;}
    | 
    TOKEN_GREAT    {$$ = 5;}
;


init_confs: glob_specs TOKEN_COLON word_specs TOKEN_SEMCOLON
{
	Init=new UCSetConf;
	Init->insert(Ref<Configuration>(new Configuration($3, $1)));
}
| init_confs glob_specs TOKEN_COLON word_specs TOKEN_SEMCOLON
{
	Init->insert(Ref<Configuration>(new Configuration($4, $2)));
};

final_confs: glob_specs TOKEN_COLON word_specs TOKEN_SEMCOLON
{
	Final=new UCSetConf;
	Final->insert(Ref<Configuration>(new Configuration($3, $1)));
}
| final_confs glob_specs TOKEN_COLON word_specs TOKEN_SEMCOLON
{
	Final->insert(Ref<Configuration>(new Configuration($4, $2)));
};

glob_specs: 
{
	$$ = new Global_Symbol(&Global_Definition);
}
| ID TOKEN_EQUAL INT
{
	if(Global_Definition.find(*$1)== Global_Definition.end())
		yyerror("error: undefined variable ");	  

	if(Global_Definition.find(*$1)!= Global_Definition.end() & !Global_Definition.in_range(*$1, $3))
		yyerror("warning: variable out of range ");	 
	
	$$ = new Global_Symbol(&Global_Definition);
	$$->set_variable(*$1, $3);
	delete($1);
}
| glob_specs TOKEN_AND ID TOKEN_EQUAL INT
{
	if(Global_Definition.find(*$3)== Global_Definition.end())
		yyerror("error: undefined variable ");	  

	if(Global_Definition.find(*$3)!= Global_Definition.end() & !Global_Definition.in_range(*$3, $5))
		yyerror("warning: variable out of range ");	 

	$$->set_variable(*$3, $5);
	delete($3);
};



word_specs: proc_specs
{
	$$ = new PWord;
	$$->push_back(*$1);
	delete($1);
}
| word_specs TOKEN_COMMA proc_specs
{
	$$=$1;
	$$->push_back(*$3);
	delete($3);
};


proc_specs:
{
	$$ = new Process(&Process_Definition);
}
| ID TOKEN_EQUAL INT
{
	$$ = new Process(&Process_Definition);
	$$->set_variable(*$1, $3);
	delete($1);
}
| proc_specs TOKEN_AND ID TOKEN_EQUAL INT
{
	if(Process_Definition.find(*$3)== Process_Definition.end())
		yyerror("error: undefined variable ");	  
	if(Process_Definition.find(*$3)!= Process_Definition.end() & !Process_Definition.in_range(*$3, $5))
		yyerror("warning: variable out of range ");

	$$ = $1;
	$$->set_variable(*$3, $5);
	delete($3);
} 
;

%%
void yyerror(const char* s)
{
	std::cerr<<s<<" at line "<<line_no<<std::endl;
}





