/*
    Copyright (C) 2005 Ahmed Rezine and Noomene Ben Henda

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information: Ahmed Rezine <Rezine.Ahemd@it.uu.se> and Noomene Ben Henda <Noomene.BenHenda@it.uu.se>
*/

/**@file starpre.h
 *ADT for pre and posts over constraints.
 *
 *@author Ahmed Rezine and Noomene Ben-Henda
 */

#ifndef STARPRE_H
#define STARPRE_H

#include "transition_operation.h"
#include "transition.h"
#include "configuration.h"
#include "process.h"
#include "exconst.h"

class StarPre:public virtual Pre
{
 public:
  StarPre(const Transitions* _trans):trans(_trans){}
    
    UCSetConf operator()(const Configuration& conf) const
    {
      UCSetConf W;
      const process::PWord* w=dynamic_cast<const process::PWord*>(conf.get_word());
      const process::Global_Symbol* gs=dynamic_cast<const process::Global_Symbol*>(conf.get_global());


      //std::cout << "transitions size " << trans->size() <<  std::endl;

      for(unsigned int i=0;i<trans->size();++i)
	{
	  process::PWord::const_iterator iter=w->begin(),iter_end=w->end();
	  for(;iter!=iter_end;++iter)
	    {
	      Transition::backward_iterator bc=(*trans)[i]->fire_backward(w,iter,gs);
	      while(!bc.end())
		{
		  GExConst temp=*bc;
		  PWord* pw=new PWord(temp.word.word);
		  Global_Symbol* pgs=new Global_Symbol(temp.gs);
		  Configuration* pconf=new Configuration(pw,pgs);
		  //		  Ref<Configuration> ref_pconf=Ref<Configuration> (&Configuration(&temp.word.word,&temp.gs));
		  //std::cout << "from " << conf <<  std::endl;
		  //std::cout <<"pointing at " << *iter << std::endl;
		  //std::cout <<"transition: " << *((*trans)[i]) << std::endl;
		  //std::cout << "gives: " << *pconf << std::endl << std::endl;
		  W.insert(Ref<Configuration>(pconf));
		  //W.insert(ref_pconf);
		  ++bc;
		}
	     
	    }
	}
      return W;
    }
    
 private:
    const Transitions* trans;

};

class StarPost:public virtual Post
{
  public:
  StarPost(const Transitions* _trans):trans(_trans){}
    
    UCSetConf operator()(const Configuration& conf) const
    {
      UCSetConf W;
      const process::PWord* w=dynamic_cast<const process::PWord*>(conf.get_word());
      const process::Global_Symbol* gs=dynamic_cast<const process::Global_Symbol*>(conf.get_global());

      process::PWord::const_iterator iter=w->begin(),iter_end=w->end();

      for(;iter!=iter_end;++iter)
	{
	  for(unsigned int i=0;i<trans->size();++i)
	    {
	      Transition::forward_iterator fo=(*trans)[i]->fire_forward(w,iter,gs);
	      while(!fo.end())
		{
		  GExConst temp=*fo;
		  PWord* pw=new PWord(temp.word.word);
		  Global_Symbol* pgs=new Global_Symbol(temp.gs);
		  Configuration* pconf=new Configuration(pw,pgs);
		  W.insert(Ref<Configuration>(pconf));
		  ++fo;
		}
	    }
	}
      return W;
    }

 private:
    const Transitions* trans;

};

#endif
