#include "process.h"
#include "variable.h"
#include "starpredicate.h"
#include "starquantifier.h"
#include "condition.h"
#include "guard.h"
#include "rtransform.h"
#include "my_functional.h"
#include "task.h"
#include "action.h"
#include "transition.h"

#include <iostream>

using namespace std;
using namespace process;
using namespace variable;
using namespace starpredicate;
using namespace starquantifier;
using namespace guard;
using namespace my_stl;
using namespace rtransform;
using namespace action;

Variable_Table Process_Definition,Global_Definition;
Process PF1(&Process_Definition),PF2(&Process_Definition),PI1(&Process_Definition);
Global_Symbol GF(&Global_Definition),GI(&Global_Definition);

bool test_process()
{
  Process_Definition.define_variable("pc",1,7,true);
  Process_Definition.define_variable("w",0,1,true);
  Process_Definition.define_variable("s",0,1,true);
 
  Global_Definition.define_variable("g", 1,2, true);
  GF["g"]=2;

  GI["g"]=2;

  PI1["pc"]=1;
  PI1["s"]=2;
  PI1["w"]=2;

  PF1["pc"]=5;
  PF1["s"]=2;
  PF1["w"]=0;

  PF2["pc"]=7;
  PF2["s"]=2;
  PF2["w"]=0;

  return ((PI1.star_enabled("s")) && !(PF1 == PF2));
}


bool test_transition()
{


  RAssignment<Process> A1("w",0);
  TLocal_Task tas1(&A1);
  RAssignment<Process> A2("pc",5);
  TLocal_Task tas2(&A2);
  RAssignment<Process> A3("s",1);
  TLocal_Task tas3(&A3);

  Action Act;
  Act.push_back(&tas1);
  Act.push_back(&tas2);
  Act.push_back(&tas3);

  NEqual neq;
  Equal eq;
  StarConstant<Process> C1(4);//! constant=4
  StarConstant<Process> C2(1);//! constant=1
  StarVariable<Process> Vpc("pc");//! pc
  StarVariable<Process> Vs("s");//! s
  StarVariable<Process> Vw("w");//! w
  
  StarIntOpPredicate<Process> Pr1(&C1,&Vpc,&eq);//! pc==4?
  StarIntOpPredicate<Process> Pr2(&C2,&Vw,&neq);//! w!=1?
  StarIntOpPredicate<Process> Pr3(&C2,&Vs,&eq);//! s==1?

  StarAndPredicate<Process> Pr4(&Pr2,&Pr3);

  StarItExistsA<Process,PWord::const_iterator> Pr5(&Pr4);

  Guard G;
  G.push_back(new TLocal_Condition(&Pr1));
  G.push_back(new TGlobal_Condition_Quantified(&Pr5));
  
  Transition tr(&G, &Act); 

  cout << "Transition: " << tr << endl;

  PWord pword;
  pword.push_back(PF1);
  pword.push_back(PF2);

  cout << "working word: " << pword << endl;

  cout << "global symbol: " << GF << endl;

  cout << "firing forward: " << endl;

  Transition::forward_iterator fo=tr.fire_forward(&pword, pword.begin(), &GF);
  for(;!fo.end();++fo)
    {
      cout << (*fo).word.word << endl;
      cout << (*fo).gs << endl;
    }
  
  cout << "firing backward: " << endl;
  
  Transition::backward_iterator ba=tr.fire_backward(&pword, pword.begin(), &GF);
  for(;!ba.end();++ba)
    {
      cout << (*ba).word.word << endl;
      cout << (*ba).gs << endl;
    }
  
  return true;
}

int main()
{
  struct
  {
    string name;
    bool (*test_f)(void);
  }

  tests[] =
    {
      {"Process", test_process},
      {"Transition", test_transition}
    };

  unsigned int i;
  string result;

  for (i = 0; i < sizeof(tests)/sizeof(tests[0]); i++) {
    cout <<tests[i].name << "... ";
    result = tests[i].test_f() == true ? "Ok" : "Fail";
    cout << result << endl;
  }

  return 0;
}


