#include "process.h"
#include "configuration.h"
#include "predicate.h"
#include "variable.h"
#include "starpredicate.h"
#include "starquantifier.h"
#include "creator.h"
#include "condition.h"
#include "guard.h"
#include "transform.h"
#include "rtransform.h"
#include "my_functional.h"
#include "task.h"
#include "action.h"
#include "transition.h"
#include "starpre.h"
#include "definition.h"

#include <iostream>

using namespace std;
using namespace process;
using namespace predicate;
using namespace variable;
using namespace starpredicate;
using namespace starquantifier;
using namespace guard;
using namespace my_stl;
using namespace transform;
using namespace rtransform;
using namespace action;
using namespace creator;

Variable_Table Process_Definition,Global_Definition;
Process PI1(&Process_Definition),PI2(&Process_Definition),PF1(&Process_Definition),PF2(&Process_Definition),PF3(&Process_Definition),PF4(&Process_Definition);
PWord WI,WF,WT;
Global_Symbol GF(&Global_Definition),GI(&Global_Definition);

bool test_process()
{
  Process_Definition.define_variable("pc",1,3,false);
  Process_Definition.define_variable("f",0,1,true);
  Process_Definition.define_variable("t",1,3,true);
 
  Global_Definition.define_variable("g", 1,3, true);
  GF.set_variable("g", 2);
  cout << "global symbols here: " << GF << endl;

  GI.set_variable("g", 2);

  PI1.set_variable("pc",1);
  PI1.set_variable("f",0);
  PI1.set_variable("t",3);

  PI2.set_variable("pc",1);
  PI2.set_variable("f",0);
  PI2.set_variable("t",3);

  PF1.set_variable("pc",3);
  PF1.set_variable("f",2);
  PF1.set_variable("t",1);

  PF2.set_variable("pc",1);
  PF2.set_variable("f",1);
  PF2.set_variable("t",1);

  PF3.set_variable("pc",3);
  PF3.set_variable("f",0);
  PF3.set_variable("t",2);

  PF4.set_variable("pc",3);
  PF4.set_variable("f",2);
  PF4.set_variable("t",4);

  return ((PI1.star_enabled("t")) && (PI1 == PI2));
}

bool test_word()
{
  WF.push_back(PF1);WF.push_back(PF2);
  WI.push_back(PI1);WI.push_back(PI2);

  return !(WI <= WF);
}

bool test_configuration()
{
  PWord* wi=new PWord(WI);
  PWord* wf=new PWord(WF);
  
  Global_Symbol* gf=new Global_Symbol(GF);
  Global_Symbol* gi=new Global_Symbol(GI);

  Configuration CI(wi,gi);
  Configuration CF(wf,gf);

  return !ConfOrder()(CI,CF);
}


bool test_predicate()
{
  bool a=True()(PI1);//!  true

  False f;//! false

  Equal eq;

  Or ou;

  bool b=!f(PI2);//!  !false

  Constant C1(3);//! constant=3

  Variable<Process> V1("pc");//! pc

  Constant * pc1=new Constant(3);

  //  Variable<Process>* pv1=new 
  IntOpPredicate Pr1(pc1,&V1,&eq);//! pc==3?

  bool c=Pr1(PF2);

  ItExistsR<PWord::const_iterator,Predicate> Q1(&Pr1);//! exists p to the right | p.pc==3?

  bool d=Q1(WI.begin(),WI.end(),WI.begin());

  return a && b && !c && d;
}

bool test_guard()
{

  WF.push_back(PF3);WF.push_back(PF4);

  Equal eq;

  Constant C1(3);//! constant=3
  Variable<Process> V1("pc");//! pc
  IntOpPredicate Pr1(&C1,&V1,&eq);//! pc==3?
  
  Constant C2(0);//! constant=0
  Variable<Process> V2("f");//! f
  IntOpPredicate Pr2(&C2,&V2,&eq);//! f==0?

  And an;

  BinOpPredicate Pr3(&Pr1,&Pr2,&an);
  //! pc==3 && f==0 ?

  ItExistsR<PWord::const_iterator,Predicate> Q1(&Pr3);
  //!cond1: exists p to the right | p.pc==3 && p.f==0?
  
  ItExistsL<PWord::const_iterator,Predicate> Q2(&Pr3);
  //!cond2: exists p to the left | p.pc==3 && p.f==0?

  TGlobal_Condition_Quantified Cond1(&Q1);
  TGlobal_Condition_Quantified Cond2(&Q2);

  Guard G;//!guard: cond1 && cond2

  G.push_back(&Cond1);
  G.push_back(&Cond2);
  
  PWord::iterator iter=WF.begin();

  bool a=G(&WF,iter,&GF);++iter; 
  bool b=G(&WF,iter,&GF);++iter;
  bool c=G(&WF,iter,&GF);++iter;
  bool e=G(&WF,iter,&GF);

  return a && b && c && e;
}


bool test_transform()
{

  Assignment<Process> A("f",0);//! f=0
  Constant C(1);//! constant=1
  Variable<Process> V("f");//! f

  Equal eq;

  IntOpPredicate Pr(&C,&V,&eq);//! f==1?

  Sequence_Transform_IfL<Process,PWord::const_iterator> S(&Pr,&A);

  PWord SofWF(S(WF.begin(),WF.end(),WF.end()).word); 

  ItExistsA<PWord::const_iterator,Predicate> EPr(&Pr);

  bool a=EPr(SofWF.begin(),SofWF.end(),SofWF.begin());

  return a;
}


bool test_rtransform()
{
  Variable<Process> V("t");//! t
  Constant C(3);//! 3

  Equal eq;

  RAssignment<Process> A("t",3);//! t:= 3

  IntOpPredicate Pr(&V,&C,&eq);//! t==3?
  
  RAssignment<Process>::reverse_iterator r=A.begin(PF4);
  
  bool a=Pr(*r);

  return a; 
}


bool test_starpredicate()
{
  StarVariable<Process> V1("f");//! f
  StarConstant<Process> C1(1);//! 1
  NEqual neq;

  StarIntOpPredicate<Process> P1(&V1,&C1,&neq);//! not(f==1)?
  
  StarVariable<Process> V2("t");//! t
  StarConstant<Process> C2(1);//! 1
  StarIntOpPredicate<Process> P2(&C2,&V2,&neq);//! not(t==1)?
  
  StarAndPredicate<Process> P3(&P2,&P1);//! not(f==1) && not(t==1) ?

  StarTrue<Process> P4;

  StarVariable<Process> V3("pc");//! pc
  StarConstant<Process> C3(3);//! 3
  Equal eq;

  StarIntOpPredicate<Process> P5(&C3,&V3,&eq);//! pc==3 ?

  StarAndPredicate<Process> P6(&P5,&P3);//! (pc==3 && (not(f==1) || not(t==1))?

  StarIntOpPredicate<Process> P7(&V3,&C2,&neq);//! not(pc==1) ?
  StarConstant<Process> C4(3);
  StarIntOpPredicate<Process> P8(&V2,&C4,&eq);//! t==3 ?

  StarAndPredicate<Process> P9(&P8,&P7);//! not(pc==1) && t==3 ?
  
  StarOrPredicate<Process> P10(&P9,&P6);//! (not(pc==1) && t==3) || ((pc==3 && (not(f==1) || not(t==1))) ?
  
  cout << "Process: " << PF4 << endl;

  cout << "true: " << endl;
  StarTrue<Process>::constraint_iterator m1=P4.begin(PF4);
 
  for(;!m1.end();++m1)
    {
      cout << *m1 << endl;
    }

  cout << "t!=1: " << endl;
  StarIntOpPredicate<Process>::constraint_iterator m2=P2.begin(PF4);

   for(;!m2.end();++m2)
    {
      cout << *m2 << endl;
    }

  cout << "t!=1 && f!=1: " << endl;
  StarAndPredicate<Process>::constraint_iterator m3=P3.begin(PF4);
 
  for(;!m3.end();++m3)
    {
      cout << *m3 << endl;
    }

  cout << "pc=3 && (t!=1 && f!=1): " << endl;
  StarAndPredicate<Process>::constraint_iterator m4=P6.begin(PF4);
 
  for(;!m4.end();++m4)
    {
      cout << *m4 << endl;
    }

  cout << "(pc!=1 && t==3) || (pc=3 && (t!=1 && f!=1)): " << endl;
  StarOrPredicate<Process>::constraint_iterator m5=P10.begin(PF4);

  for(;!m5.end();++m5)
    {
      cout << *m5 << endl;
    }

  return true;
}

bool test_starquantifier()
{
  WT.push_back(PI1);WT.push_back(PI2);WT.push_back(PF4);
  //  WT.push_back(PF1);WT.push_back(PF4);WT.push_back(PF2);

  NEqual neq;
  StarVariable<Process> V1("t");//! t
  StarVariable<Process> V2("pc");//! pc
  StarConstant<Process> C1(2);//! 1

  StarIntOpPredicate<Process> P1(&C1,&V1,&neq);//! not(t==2)?
  StarIntOpPredicate<Process> P2(&C1,&V2,&neq);//! not(pc==2)?

  StarAndPredicate<Process> PStar(&P1,&P2);

  Equal eq;
  StarIntOpPredicate<Process> P11(&C1,&V1,&eq);//! t==2?
  StarIntOpPredicate<Process> P22(&C1,&V2,&eq);//! pc==2?

  StarAndPredicate<Process> PStarStar(&P11,&P22);

  //StarForAllL<Process,PWord::const_iterator> Q1(&PStarStar);
  StarForAllA<Process,PWord::const_iterator> Q1(&P1);
  //  StarItExistsR<Process,PWord::const_iterator> Q2(&PStarStar);
  StarItExistsL<Process,PWord::const_iterator> Q2(&PStarStar);

  StarAndPredicate<Process>::constraint_iterator m=PStar.begin(PF4);
  StarAndPredicate<Process>::constraint_iterator mm=PStarStar.begin(PF4);
  /*
  cout << "work process :" << PF4 << endl;
  cout << PStar << endl;
  for(;!m.end();++m)
    {
      cout << *m << endl;
    }

    
  cout << "PR2=pc==2 || t==2:" << endl;
  for(;!mm.end();++mm)
    {
      cout << *mm << endl;
    }
  
  
  StarForAllA<Process,PWord::const_iterator>::constraint_iterator mf=Q1.begin(WT.begin(),WT.end(),++++++WT.begin());
 
  cout << "work word w: " << endl << WT << endl;
  cout << "pointing at " << *WT.begin() << endl;
  cout << Q1 << endl;
  for(;!mf.end();++mf)
    {
      cout << (*mf).word << endl;
    }
  */
  Creator<Process> c(&PStarStar);
  Creator<Process>::iterator iter=c.begin();

  cout << "creator example for " << PStarStar << endl;
  for(;!iter.end();++iter)
    {
      cout << *iter << endl;
    }

  //  StarItExistsL<Process,PWord::const_iterator>::constraint_iterator me=Q2.begin(WT.begin(),WT.end(),++++++WT.begin());
  StarItExistsL<Process,PWord::const_iterator>::constraint_iterator me=Q2.begin(WT.begin(),WT.end(),WT.begin());

  cout << Q2 << endl;
  for(;!me.end();++me)
   {
     cout << (*me).word << endl;
   }
  
  return true;
}

bool test_condition()
{
  StarVariable<Process> V1("f");//! f
  StarConstant<Process> C1(1);//! 1
  NEqual neq;

  StarIntOpPredicate<Process> P1(&V1,&C1,&neq);//! not(f==1)?
  
  StarVariable<Process> V2("t");//! t
  StarConstant<Process> C2(1);//! 1
  StarIntOpPredicate<Process> P2(&C2,&V2,&neq);//! not(t==1)?
  
  StarAndPredicate<Process> P3(&P2,&P1);//! not(f==1) && not(t==1) ?

  StarTrue<Process> P4;

  StarVariable<Process> V3("pc");//! pc
  StarConstant<Process> C3(3);//! 3
  Equal eq;

  StarIntOpPredicate<Process> P5(&C3,&V3,&eq);//! pc==3 ?

  StarAndPredicate<Process> P6(&P5,&P3);//! (pc==3 && (not(f==1) || not(t==1))?

  StarIntOpPredicate<Process> P7(&V3,&C2,&neq);//! not(pc==1) ?
  StarConstant<Process> C4(3);
  StarIntOpPredicate<Process> P8(&V2,&C4,&eq);//! t==3 ?

  StarAndPredicate<Process> P9(&P8,&P7);//! not(pc==1) && t==3 ?
  
  StarOrPredicate<Process> P10(&P9,&P6);//! (not(pc==1) && t==3) || ((pc==3 && (not(f==1) || not(t==1))) ?

  StarOrPredicate<Process>::constraint_iterator m=P10.begin(PF4);
  
  for(;!m.end();++m)
    {
      cout << *m << endl;
    }


  TLocal_Condition cond(&P10);

  cout << "working word: " << WT << endl; 

  cout << "check condition: " << cond(&WT,++(++WT.begin()),&GF) << endl;

  TLocal_Condition::constraint_generator c=cond.begin(&WT,++(++WT.begin()),&GF);

  for(;!c.end();++c)
    {
      cout << (*c).word.word << endl;
    }

  return true;
}

bool test_task()
{
  RAssignment<Process> A("f",1);
  
  TLocal_Task tas(&A);

  cout << "working word: " << WT << endl; 

  cout << "check task: " << tas(&WT,++(++WT.begin()),&GF).word.word << endl;

  TLocal_Task::reverse_generator c=tas.begin(&WT,++(++WT.begin()),&GF);

  for(;!c.end();++c)
    {
      cout << (*c).word.word << endl;
    }

  return true;
}

bool test_transition()
{


  RAssignment<Process> A1("f",1);
  TLocal_Task tas1(&A1);
  RAssignment<Process> A2("pc",3);
  TLocal_Task tas2(&A2);

  RAssignment<Process> A5("pc",1);


  RAssignment<Process> A3("t",1);
  TLocal_Task tas3(&A3);

  RAssignment<Global_Symbol> A4("g",2);
  TGlobal_Task tas4(&A4);

  Action Act;
  //  Act.push_back(&tas1);
  Act.push_back(&tas2);
  //  Act.push_back(&tas3);


  //  Act.push_back(new TGlobal_Sequence_Task(new RSequence_Transform_A<Process, PWord::const_iterator>(&A5)));

  NEqual neq;
  Equal eq;
  StarConstant<Process> C1(1);//! constant=1
  StarConstant<Process> C2(2);//! constant=2
  StarVariable<Process> Vpc("pc");//! pc
  StarVariable<Process> Vt("t");//! t
  StarIntOpPredicate<Process> Pr1(&C2,&Vpc,&eq);//! pc==2?
  StarIntOpPredicate<Process> Pr2(&C2,&Vt,&eq);//! t=2?


  Guard G;
  G.push_back(new TLocal_Condition(&Pr1));
  //G.push_back(new TLocal_Condition(&Pr2));

  //  G.push_back(new TGlobal_Condition_Quantified( new StarForAllL<Process,PWord::const_iterator>(&Pr2)));

  //G.push_back(new TGlobal_Condition_Quantified( new StarItExistsL<Process,PWord::const_iterator>(&Pr1)));

	     //  StarItExistsL<Process,PWord::const_iterator> Q2(&Pr1);
	     //G.push_back(TGlobal_Consition_Quantified(&Pr2));

  
  StarConstant<Global_Symbol> C(1);//! constant=2
  StarVariable<Global_Symbol> Vg("g");//! pc
  StarIntOpPredicate<Global_Symbol> Prg(&C,&Vg,&eq);//! pc==2?
  TGlobal_Condition condg(&Prg);


	


  //  G.push_back(&condg);
  
  Transition tr(&G, &Act); // pc!=2 => f:=1 & t:=2

  cout << "Transition: " << tr << endl;

  PWord pword;
  pword.push_back(PF2);
  pword.push_back(PF1);
  pword.push_back(PF2);


  cout << "working word: " << pword << endl;

  cout << "global symbol: " << *GF.get_definition() << endl;
  cout << "global symbol table: " << GF << endl;

  /*  cout << "firing forward: " << endl;

  Transition::forward_iterator fo=tr.fire_forward(&WT, WT.begin(), &GF);
  for(;!fo.end();++fo)
    {
      cout << (*fo).word.word << endl;
      cout << (*fo).gs << endl;
    }
  */
  
  cout << "firing backward: " << endl;
  //  Transition::backward_iterator ba=tr.fire_backward(&WT, ++(++WT.begin()), &GF);

  Transition::backward_iterator ba=tr.fire_backward(&pword,(++pword.begin()), &GF);
  for(;!ba.end();++ba)
    {
      cout << (*ba).word.word << endl;
      cout << (*ba).gs << endl;
    }
  

  return true;
}

bool test_dij_transition()
{

  //  Variable_Table Process_Definition, Global_Definition;
  Process Prot_PI1(&Process_Definition), Prot_PI2(&Process_Definition),Prot_PI3(&Process_Definition);
  PWord Prot_W;
  Global_Symbol Prot_GF(&Global_Definition);

  Process_Definition.define_variable("pc",1,7,false);
  Process_Definition.define_variable("p",0,1, false);
  Process_Definition.define_variable("f",0,2, false);
 
  Global_Definition.define_variable("g", 1,2, true);
  Prot_GF.set_variable("g", 3);

  Prot_PI1.set_variable("pc",6);
  Prot_PI1.set_variable("p",0);
  Prot_PI1.set_variable("f",0);

  Prot_PI2.set_variable("pc",6);
  Prot_PI2.set_variable("p",0);
  Prot_PI2.set_variable("f",2);

  Prot_PI3.set_variable("pc",5);
  Prot_PI3.set_variable("p",1);
  Prot_PI3.set_variable("f",3);


  Prot_W.push_back(Prot_PI1);
  Prot_W.push_back(Prot_PI2);
  //  Prot_W.push_back(Prot_PI3);

  //assignments
  RAssignment<Process> pc1("pc",1);
  TLocal_Task taspc1(&pc1);

  RAssignment<Process> pc6("pc",6);
  TLocal_Task taspc6(&pc6);

  //  TGlobal_Sequence_Task tgstp0(new RSequence_Transform_A<Process, PWord::const_iterator>
  //						     (new RAssignment<Process>("p", 0)));

  Action Actpc1, Actpc6;
  Actpc1.push_back(&taspc1);
  Actpc6.push_back(&taspc6);

  //Act.push_back(new TGlobal_Sequence_Task(new RSequence_Transform_A<Process, PWord::const_iterator>(&A5)));

  //guard
  NEqual neq;
  Equal eq;
  StarConstant<Process> C0(0);//! constant=
  StarConstant<Process> C1(1);//! constant=1
  StarConstant<Process> C2(2);//! constant=2
  StarConstant<Process> C3(3);//! constant=3
  StarConstant<Process> C5(5);//! constant=5
  StarVariable<Process> Vp("p");//! p
  StarVariable<Process> Vf("f");//! f
  StarVariable<Process> Vpc("pc");//! pc
  StarIntOpPredicate<Process> Prp1(&C1,&Vp,&eq);//! pc==2?
  StarIntOpPredicate<Process> Prf0(&C0,&Vf,&eq);//! f=0?
  StarIntOpPredicate<Process> Prpd1(&C1,&Vp,&neq);//! p!=1?
  StarIntOpPredicate<Process> Prpc2(&C2,&Vpc,&eq);//! pc=2?
  StarIntOpPredicate<Process> Prpc3(&C3,&Vpc,&eq);//! pc=3?
  StarIntOpPredicate<Process> Prpc5(&C5,&Vpc,&eq);//! pc=5?

  StarIntOpPredicate<Process> Prf2(&C2,&Vf,&eq);//! f=2?
  StarIntOpPredicate<Process> Prfd2(&C2,&Vf,&neq);//! f!=2?

  StarAndPredicate<Process> Pand(&Prp1,&Prf0);//! (p==1) && (f==0) ?
  StarOrPredicate<Process> Por(&Pand,&Prpd1);//! ((p==1) && (f==0))|(p!=1) ?
  
  //conditions
  TGlobal_Condition_Quantified exleft(new StarItExistsL<Process,PWord::const_iterator>(&Prf2));  
  TGlobal_Condition_Quantified exright(new StarItExistsR<Process,PWord::const_iterator>(&Prf2));  
  TGlobal_Condition_Quantified all(new StarForAllA<Process,PWord::const_iterator>(&Prfd2));  

  TGlobal_Condition_Quantified alll(new StarForAllL<Process,PWord::const_iterator>(&Prfd2));  
  TGlobal_Condition_Quantified allr(new StarForAllR<Process,PWord::const_iterator>(&Prfd2));  


  Guard Gl, Gr, Ga;
  Gl.push_back(new TLocal_Condition(&Prpc5));
  Gl.push_back(&exleft);

  Gr.push_back(new TLocal_Condition(&Prpc5));
  Gr.push_back(&exright);

  Ga.push_back(new TLocal_Condition(&Prpc5));
  //  Ga.push_back(&all);
  Ga.push_back(&alll);
  Ga.push_back(&allr);


  
  Transition trl(&Gl, &Actpc1); 
  Transition trr(&Gr, &Actpc1);
  Transition tra(&Ga, &Actpc6); 

  cout << "Transitionl: " << trl << endl;
  cout << "Transitionr: " << trr << endl;
  cout << "Transitiona: " << tra << endl;

  cout << "working word: " << Prot_W << endl;


  /*  cout << "firing forward: " << endl;

  Transition::forward_iterator fo=tr.fire_forward(&WT, WT.begin(), &GF);
  for(;!fo.end();++fo)
    {
      cout << (*fo).word.word << endl;
      cout << (*fo).gs << endl;
    }
  */
  cout << "firing backward: " << endl;
  /*
  Transition::backward_iterator bal=trl.fire_backward(&Prot_W,(Prot_W.begin()), &Prot_GF);
  for(;!bal.end();++bal)
    {
      cout << (*bal).word.word << endl;
    }
  cout << "#" << endl;
  Transition::backward_iterator bar=trr.fire_backward(&Prot_W,(Prot_W.begin()), &Prot_GF);
  for(;!bar.end();++bar)
    {
      cout << (*bar).word.word << endl;
    }
  */
  cout << "#" << endl;
  Transition::backward_iterator baa=tra.fire_backward(&Prot_W,(Prot_W.begin()), &Prot_GF);
  for(;!baa.end();++baa)
    {
      cout << (*baa).word.word << endl;
    }
  

  return true;
}
bool test_szy_transition()
{

  Process Prot_PI1(&Process_Definition), Prot_PI2(&Process_Definition),Prot_PI3(&Process_Definition);
  PWord Prot_W;
  Global_Symbol Prot_GF(&Global_Definition);

  Process_Definition.define_variable("pc",1,7,true);
  Process_Definition.define_variable("s",0,1,true);
  Process_Definition.define_variable("w",0,1,true);
 
  Global_Definition.define_variable("g", 1,2, true);
  Prot_GF.set_variable("g", 2);

  Prot_PI1.set_variable("pc",1);
  Prot_PI1.set_variable("s",1);
  Prot_PI1.set_variable("w",0);

  Prot_PI2.set_variable("pc",4);
  Prot_PI2.set_variable("s",2);
  Prot_PI2.set_variable("w",2);

  Prot_PI3.set_variable("pc",4);
  Prot_PI3.set_variable("s",2);
  Prot_PI3.set_variable("w",2);


  Prot_W.push_back(Prot_PI1);
  Prot_W.push_back(Prot_PI2);
  Prot_W.push_back(Prot_PI3);

  //assignments
  RAssignment<Process> w0("w",0);
  TLocal_Task tasw0(&w0);

  RAssignment<Process> s0("s",0);
  TLocal_Task tass0(&s0);

  RAssignment<Process> s1("s",1);
  TLocal_Task tass1(&s1);

  RAssignment<Process> pc4("pc",4);
  TLocal_Task taspc4(&pc4);

  RAssignment<Process> pc5("pc",5);
  TLocal_Task taspc5(&pc5);

  RAssignment<Process> pc7("pc",7);
  TLocal_Task taspc7(&pc7);

  Action Actpc34, Actpc35, Actpc45, Actpc67;
  Actpc34.push_back(&taspc4);
  Actpc34.push_back(&tass0);

  Actpc35.push_back(&taspc5);
  Actpc35.push_back(&tasw0);

  Actpc45.push_back(&taspc5);
  Actpc45.push_back(&tasw0);
  Actpc45.push_back(&tass1);

  Actpc67.push_back(&taspc7);

  //guard
  NEqual neq;
  Equal eq;
  StarConstant<Process> C0(0);//! constant=
  StarConstant<Process> C1(1);//! constant=1
  StarConstant<Process> C2(2);//! constant=2
  StarConstant<Process> C3(3);//! constant=3
  StarConstant<Process> C4(4);//! constant=4
  StarConstant<Process> C5(5);//! constant=5
  StarConstant<Process> C6(6);//! constant=6
  StarVariable<Process> Vs("s");//! s
  StarVariable<Process> Vw("w");//! w
  StarVariable<Process> Vpc("pc");//! pc

  StarIntOpPredicate<Process> Prwd1(&C1,&Vw,&neq);//! w!=1?
  StarIntOpPredicate<Process> Prw0(&C0,&Vw,&eq);//! w==0?
  StarIntOpPredicate<Process> Prw1(&C1,&Vw,&eq);//! w==1?


  StarIntOpPredicate<Process> Prpcd1(&C1,&Vpc,&neq);//! pc!=1?
  StarIntOpPredicate<Process> Prpc1(&C1,&Vpc,&eq);//! pc==1?
  StarIntOpPredicate<Process> Prpc2(&C2,&Vpc,&eq);//! pc=2?
  StarIntOpPredicate<Process> Prpc3(&C3,&Vpc,&eq);//! pc=3?
  StarIntOpPredicate<Process> Prpc4(&C4,&Vpc,&eq);//! pc=4?
  StarIntOpPredicate<Process> Prpc5(&C5,&Vpc,&eq);//! pc=5?
  StarIntOpPredicate<Process> Prpc6(&C6,&Vpc,&eq);//! pc=6?

  StarIntOpPredicate<Process> Prs1(&C1,&Vs,&eq);//! s=1?
  StarIntOpPredicate<Process> Prs0(&C0,&Vs,&eq);//! s=0?


  StarAndPredicate<Process> Pand3(&Prpcd1,&Prwd1);//! (pc!=1) && (w!=1) ?
  StarOrPredicate<Process> Ppc1orw1(&Prpc1,&Prw1);//! (pc==1) || (w==1) ?
  StarOrPredicate<Process> Ps0orw0(&Prs0,&Prw0);//! (s==0) || (w==0) ?

  StarAndPredicate<Process> Pand4(&Prs1,&Prw0);//! (s==1) && (w==0) ?

  //conditions
  TGlobal_Condition_Quantified exleft3(new StarItExistsL<Process,PWord::const_iterator>(&Pand3));  
  TGlobal_Condition_Quantified exright3(new StarItExistsR<Process,PWord::const_iterator>(&Pand3));  
  TGlobal_Condition_Quantified all3(new StarForAllA<Process,PWord::const_iterator>(&Ppc1orw1));  

  TGlobal_Condition_Quantified exleft4(new StarItExistsL<Process,PWord::const_iterator>(&Pand4));  
  TGlobal_Condition_Quantified exright4(new StarItExistsR<Process,PWord::const_iterator>(&Pand4));

  TGlobal_Condition_Quantified alll67(new StarForAllL<Process,PWord::const_iterator>(&Ps0orw0));    


  Guard Gl3, Gr3, Ga3, Gl4, Gr4, G6;
  Gl3.push_back(new TLocal_Condition(&Prpc3));
  Gl3.push_back(&exleft3);

  Gr3.push_back(new TLocal_Condition(&Prpc3));
  Gr3.push_back(&exright3);

  Ga3.push_back(new TLocal_Condition(&Prpc3));
  Ga3.push_back(&all3);

  Gl4.push_back(new TLocal_Condition(&Prpc4));
  Gl4.push_back(&exleft4);

  Gr4.push_back(new TLocal_Condition(&Prpc4));
  Gr4.push_back(&exright4);

  G6.push_back(new TLocal_Condition(&Prpc6));
  G6.push_back(&alll67);
 
  Transition trl34(&Gl3, &Actpc34); 
  Transition trr34(&Gr3, &Actpc34);

  Transition tra35(&Ga3, &Actpc35);

  Transition trl45(&Gl4, &Actpc45); 
  Transition trr45(&Gr4, &Actpc45); 

  Transition tr67(&G6, &Actpc67); 

  cout << "working word: " << Prot_W << endl;

  cout << "firing backward: " << endl;
  cout << "Transitionl: " << trl34 << endl;
  Transition::backward_iterator bal34=trl34.fire_backward(&Prot_W,(++Prot_W.begin()), &Prot_GF);
  for(;!bal34.end();++bal34)
    {
      cout << (*bal34).word.word << endl;
    }
  cout << "#" << endl;

  cout << "Transitionl: " << trr34 << endl;
  Transition::backward_iterator bar34=trr34.fire_backward(&Prot_W,(++Prot_W.begin()), &Prot_GF);
  for(;!bar34.end();++bar34)
    {
      cout << (*bar34).word.word << endl;
    }
    cout << "#" << endl; 
  
    /*
      cout << "Transitionl: " << tra35 << endl;
      Transition::backward_iterator baa35=tra35.fire_backward(&Prot_W,(++Prot_W.begin()), &Prot_GF);
  for(;!baa35.end();++baa35)
    {
      cout << (*baa35).word.word << endl;
    }
  
  cout << "#" << endl;

  cout << "Transitionl: " << trl45 << endl;
  Transition::backward_iterator bal45=trl45.fire_backward(&Prot_W,(++Prot_W.begin()), &Prot_GF);
  for(;!bal45.end();++bal45)
    {
      cout << (*bal45).word.word << endl;
    }
  cout << "#" << endl;
  
  cout << "Transitionl: " << trr45 << endl;
  Transition::backward_iterator bar45=trr45.fire_backward(&Prot_W,(++Prot_W.begin()), &Prot_GF);
  for(;!bar45.end();++bar45)
    {
      cout << (*bar45).word.word << endl;
    }
  
  cout << "Transition: " << tr67 << endl;
  Transition::backward_iterator ba67=tr67.fire_backward(&Prot_W,++(++Prot_W.begin()), &Prot_GF);
  for(;!ba67.end();++ba67)
    {
      cout << (*ba67).word.word << endl;
    }
    */

  return true;
}

bool test_new()
{
  
  RAssignment<Process>* A1=new RAssignment<Process>("f",1);
  TLocal_Task* tas1=new TLocal_Task(A1);
  RAssignment<Process> A2("pc",3);
  TLocal_Task tas2(&A2);

  RAssignment<Process> A5("pc",1);
  TLocal_Task tas5(&A5);

  RAssignment<Process> A3("t",1);
  TLocal_Task tas3(&A3);

  RAssignment<Global_Symbol> A4("g",2);
  TGlobal_Task tas4(&A4);

  Action* Act=new Action;
  Act->push_back(tas1);
  //Act.push_back(&tas5);
  Act->push_back(&tas4);


  //  Act.push_back(new TGlobal_Sequence_Task(new RSequence_Transform_A<Process, PWord::const_iterator>(&A5)));

  NEqual neq;
  Equal eq;
  StarConstant<Process> C1(1);//! constant=1
  StarConstant<Process> C2(2);//! constant=2
  StarVariable<Process> Vpc("pc");//! pc
  StarVariable<Process> Vt("t");//! t
  StarIntOpPredicate<Process>* Pr1=new StarIntOpPredicate<Process>(&C2,&Vpc,&eq);//! pc==2?
  StarIntOpPredicate<Process>* Pr2=new StarIntOpPredicate<Process>(&C2,&Vt,&eq);//! t=2?


   
  StarConstant<Global_Symbol> C(1);//! constant=2
  StarVariable<Global_Symbol> Vg("g");//! pc
  StarIntOpPredicate<Global_Symbol> Prg(&C,&Vg,&eq);//! pc==2?
  TGlobal_Condition condg(&Prg);

  TNew_Condition tnew();

  Guard G;
  //  G.push_back(new TLocal_Condition(&Pr1));
  G.push_back(&condg);
  G.push_back(new TNew_Condition());
  //G.push_back(new TLocal_Condition(&Pr2));
  
  //  G.push_back(new TGlobal_Condition_Quantified( new StarForAllL<Process,PWord::const_iterator>(&Pr2)));

  //G.push_back(new TGlobal_Condition_Quantified( new StarItExistsL<Process,PWord::const_iterator>(&Pr1)));

	     //  StarItExistsL<Process,PWord::const_iterator> Q2(&Pr1);
	     //G.push_back(TGlobal_Consition_Quantified(&Pr2));

 

	


  //  G.push_back(&condg);
  
  Transition* tr=new Transition(&G, Act); // pc!=2 => f:=1 & t:=2
  
  cout << "Transition: " << *tr << endl;

  //Transitions* trans=new Transitions;
  //trans->push_back(&tr);
  //StarPre spre(trans);


  PWord pword;
  pword.push_back(PF2);
  pword.push_back(PF1);
  pword.push_back(PF2);

  PWord* pw=new PWord(pword);
  Global_Symbol* pgs=new Global_Symbol(GF);

  //  Configuration* pconf=new Configuration(pw, pgs);



  cout << "working word: " << pword << endl;

  cout << "global symbol: " << GF << endl;

  //UCSetConf Wt;
  //  Wt=spre.operator()(*pconf);

   /*
  cout << "firing forward: " << endl;

  Transition::forward_iterator fo=tr.fire_forward(&WT, WT.begin(), &GF);
  for(;!fo.end();++fo)
    {
      cout << (*fo).word.word << endl;
      cout << (*fo).gs << endl;
    }
  */
   
  cout << "firing backward: 123 .... " << endl;
  cout << " pointing at " << *((++++pword.begin())) << endl; 
  Transition::backward_iterator ba=tr->fire_backward(&pword,(++++pword.begin()), &GF);
  for(;!ba.end();++ba)
    {
      cout << (*ba).word.word << endl;
      cout << (*ba).gs << endl;
    }
  
  cout << "here" << endl;
  return true;
}


int main()
{
  struct
  {
    string name;
    bool (*test_f)(void);
  }

  tests[] =
    {
      {"Process", test_process},
      //{"Word", test_word},
      //{"Configuration", test_configuration},
      //{"Predicates", test_predicate}
      //{"Guards", test_guard},
      //{"Transform", test_transform},
      //{"Reversible Transform", test_rtransform},
      //{"Star Predicate", test_starpredicate},
      //      {"Star Quantifier", test_starquantifier}
      //{"Condition", test_condition},
      //{"Task", test_task},
      //{"Transition", test_transition},
      //{"Prot dij Transition", test_dij_transition}
      //{"Prot szy Transition", test_szy_transition}
      {"New", test_new}
    };

  unsigned int i;
  string result;

  for (i = 0; i < sizeof(tests)/sizeof(tests[0]); i++) {
    cout <<tests[i].name << "... ";
    result = tests[i].test_f() == true ? "Ok" : "Fail";
    cout << result << endl;
  }

  return 0;
}


