/*
    Copyright (C) 2005 Ahmed Rezine and Noomene Ben Henda

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information: Ahmed Rezine <Rezine.Ahemd@it.uu.se> and Noomene Ben Henda <Noomene.BenHenda@it.uu.se>
*/

/**@file tlist.h
 *ADT for tlists.
 *
 *@author Ahmed Rezine and Noomene Ben-Henda
 */

#ifndef TLIST_H
#define TLIST_H
#include <list>
#include "printable.h"

/**
 *class ListT.
 *
 */
template<class T>
class TList:private std::list<const T*>, public virtual Printable
{
 public:
  typedef std::list<const T*> ContainerT;

 public:
  TList() {}
  ~TList(){
    const_iterator iter=begin(),iter_end=end();
    while(iter != iter_end)
      {
	delete(*iter);
	++iter;
      }
  };

 public:
  typedef typename ContainerT::const_iterator const_iterator;
  typedef typename ContainerT::iterator iterator;

  /**iterators*/
  iterator begin() { return ContainerT::begin(); }
  const_iterator begin() const { return ContainerT::begin(); }

  iterator end() { return ContainerT::end(); }
  const_iterator end() const { return ContainerT::end(); }
  
  /**operations*/  
  void swap(TList<T> other) { ContainerT::swap(other); }
  void clear() { ContainerT::clear(); }

  void erase(iterator it) { ContainerT::erase(it); }
  void erase(const_iterator it) { ContainerT::erase(it); }

  /**addition*/
  void push_back(const T* x) { ContainerT::push_back(x); }
  void push_front(const T* x) { ContainerT::push_front(x); }
  void insert(iterator pos, const T* x) { ContainerT::insert(pos,x); }

  /**others*/
  bool empty() const { return ContainerT::empty(); }

  
  /**inherited from Printable*/
  void print(std::ostream& out) const
  {
    const_iterator iter=begin(),iter_end=end();
    while(iter != iter_end)
      {
	out << **iter;++iter;
	if (iter != iter_end) {out << " & ";}
      }
  }

};

#endif

