/*
    Copyright (C) 2005 Ahmed Rezine and Noomene Ben Henda

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information: Ahmed Rezine <Rezine.Ahemd@it.uu.se> and Noomene Ben Henda <Noomene.BenHenda@it.uu.se>
*/

/**@file transiton.h
 *A transition is basically a guard and an action.
 *
 *@author Ahmed Rezine and Noomene Ben-Henda
 */

#ifndef TRANSITION_H
#define TRANSITION_H

#include "guard.h"
#include "action.h"
#include "ref.h"
#include "process.h"
#include "utilities.h"
#include "printable.h"
#include "exconst.h"

#include <vector>

using namespace guard;
using namespace action;

/**
 *class transition.
 */
class Transition:public virtual Printable
{
 public:
  Transition(const Guard* _guard,const Action* _action):guard(_guard),action(_action) {}
    ~Transition(){delete(guard);delete(action);}
 public:
  class backward_iterator
  {
    typedef GExConst value_type;

  public:
    backward_iterator() {}
    backward_iterator(const Transition* _tr, const process::PWord* _w, const process::PWord::const_iterator& _pos, const process::Global_Symbol* _gs):tr(_tr),w(_w),pos(_pos),gs(_gs),e(false) 
      {
	if (!Compose<Task>().build_comp_table<Action::const_iterator>(task_comp_table,tr->get_action()->begin(),tr->get_action()->end(),w,pos,gs)) {e=true;}
	else 
	  {
	    g=*task_comp_table[0].second.second;
	    while (!Compose<Condition>().build_comp_table<Guard::const_iterator>(cond_comp_table,tr->get_guard()->begin(),tr->get_guard()->end(),&g.word.word,g.word.curr,&g.gs) && !e) 
	      {
		if (!Compose<Task>().update_up(task_comp_table,0)) {e=true;}
		else {g=*task_comp_table[0].second.second;}
	      }
	  }
      }

      bool end() const {return e;}
     
      backward_iterator& operator=(const  backward_iterator& other)
	{
	  tr=other.tr;w=other.w;pos=other.pos;gs=other.gs;e=other.e;g=other.g;
	  task_comp_table=other.task_comp_table;
	  cond_comp_table=other.cond_comp_table;
	  return *this;
	}

      backward_iterator& operator++()
	{
	  if (!e)
	    {
	      if (!Compose<Condition>().update_up(cond_comp_table,0))
		{
		  if (Compose<Task>().update_up(task_comp_table,0)) 
		    {
		      g=*task_comp_table[0].second.second;
		      while (!Compose<Condition>().build_comp_table<Guard::const_iterator>(cond_comp_table,tr->get_guard()->begin(),tr->get_guard()->end(),&g.word.word,g.word.curr,&g.gs) && !e) 
			{
			  if (!Compose<Task>().update_up(task_comp_table,0)) {e=true;return *this;}
			  else {g=*task_comp_table[0].second.second;}
			}
		    }
		  else {e=true;return *this;}
		}
	    }
	  return *this;
	}

      value_type operator*() const
      {
	if (!e) {return *cond_comp_table[0].second.second;}
	return g;
      }

  private:
      const Transition* tr;
      const process::PWord* w;
      process::PWord::const_iterator pos;
      const Global_Symbol* gs;
      bool e; 
      Compose<Task>::vec_type task_comp_table;
      value_type g;
      Compose<Condition>::vec_type cond_comp_table;
  };

  class forward_iterator
  {
    typedef GExConst value_type;
  public:
    forward_iterator() {}
    forward_iterator(const Transition* _tr, const process::PWord* _w, const process::PWord::const_iterator& _pos, const process::Global_Symbol* _gs):tr(_tr),w(_w),pos(_pos),gs(_gs),e(false) 
      {
	if (!tr->get_guard()->operator()(w,pos,gs)) {e=true;}
      }

      bool end() const {return e;}
     
      forward_iterator& operator=(const  forward_iterator& other)
	{
	  tr=other.tr;w=other.w;pos=other.pos;gs=other.gs;e=other.e;
	  return *this;
	}

      forward_iterator& operator++()
	{
	  if (!e) {e=true;}
	  return *this;
	}

      value_type operator*() const
      {
	return tr->get_action()->operator()(w,pos,gs);
      }

  private:
      const Transition* tr;
      const process::PWord* w;
      process::PWord::const_iterator pos;
      const Global_Symbol* gs;
      bool e; 
  };

  const Guard* get_guard() const {return guard;}

  const Action* get_action() const {return action;}

  backward_iterator fire_backward(const process::PWord* w, const process::PWord::const_iterator& pos, const process::Global_Symbol* gs) const
    {
      return backward_iterator(this,w,pos,gs);
    }

  forward_iterator& fire_forward(const process::PWord* w, const process::PWord::const_iterator& pos, const process::Global_Symbol* gs) const
    {
      return *new forward_iterator(this,w,pos,gs);
    }

  void print(std::ostream& out) const
  {
    out << "{ Guard: " << *guard << " | Action: " << *action << " }";
  }

 protected:
  const Guard* guard;
  const Action* action;
};

class Transitions:public std::vector<const Transition*>,public virtual Printable
{
 public:
  typedef std::vector<const Transition*> base_type;
  typedef base_type::iterator iterator;
  typedef base_type::const_iterator const_iterator;

 public:
  Transitions() {}

  ~Transitions(){
   for(std::vector<const Transition*>::const_iterator it=begin(); it!=end(); it++)
     delete(*it);
  }

  void print(std::ostream& out) const
  {
    for(std::vector<const Transition*>::const_iterator it=begin(); it!=end(); it++)
      out << *it;
  }
  
};

#endif
