/*
    Copyright (C) 2005 Ahmed Rezine and Noomene Ben Henda

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information: Ahmed Rezine <Rezine.Ahemd@it.uu.se> and Noomene Ben Henda <Noomene.BenHenda@it.uu.se>
*/

/**@file transition_relation.h
 *Here are defined various operators related to the transition relation such as pre, 
 *post, exact_pre ... 
 *
 *@author Ahmed Rezine and Noomene Ben-Henda
 */
#ifndef TRANSITION_OPERATION_H
#define TRANSITION_OPERATION_H

#include "configuration.h"

/**
 *This class implements the pre operator.
 */

class Pre//:public std::unary_function<ConfRef,UCSetConf>
{
 public:
  virtual UCSetConf operator()(const Configuration& conf) const=0;
  
  UCSetConf operator()(const ConfRef conf) const
  {
    return operator()(*conf);
  }
  
  UCSetConf operator()(const UCSetConf& ucsconf) const
  {
    UCSetConf V,W;
    UCSetConf::const_iterator p=ucsconf.begin(),p_end=ucsconf.end();
    for(; p!=p_end ; p++)
      {
	W=operator()(*p);
	V.insert(W.begin(),W.end());
	W.clear();
      }
    return V;
  }

  virtual ~Pre() {}

};

/**
 *This class implements the post operator.
 */

class Post
{
 public:
  virtual UCSetConf operator()(const Configuration& conf) const=0;
  UCSetConf operator()(const ConfRef conf) const
  {
    return operator()(*conf);
  }
  UCSetConf operator()(const UCSetConf& ucsconf) const
  {
    UCSetConf V,W;
    UCSetConf::const_iterator p=ucsconf.begin(),p_end=ucsconf.end();
    for(; p!=p_end ; p++)
      {
	W=operator()(*p);
	V.insert(W.begin(),W.end());
	W.clear();
      }
    return V;
  }

  virtual ~Post() {}

};

/**
 *This class implements the exact_pre operator.
 */
class ExactPre
{
  
public:
  virtual SetConf operator()(const Configuration& conf) const=0;
  SetConf operator()(const ConfRef conf) const
  {
    return operator()(*conf);
  }
  SetConf operator()(const SetConf& sconf) const
  {
    SetConf V,W;
    SetConf::const_iterator p=sconf.begin(),p_end=sconf.end();
    for(; p!=p_end ; p++)
      {
	W=operator()(*p);
	V.insert(W.begin(),W.end());
	W.clear();
      }
    return V;
  }
  virtual ~ExactPre() {}

};

/**
 *This class implements the exact_post operator.
 */
 
class ExactPost
{

public:
  virtual SetConf operator()(const Configuration& conf) const=0;
  SetConf operator()(const ConfRef conf) const
  {
    return operator()(*conf);
  }
  SetConf operator()(const SetConf& sconf) const 
  {
    SetConf V,W;
    SetConf::const_iterator p=sconf.begin(),p_end=sconf.end();
    for(; p!=p_end ; p++)
      {
	W=operator()(*p);
	V.insert(W.begin(),W.end());
	W.clear();
      }
    return V;
  }
  virtual ~ExactPost() {}

};

#endif

