/*
    Copyright (C) 2005 Ahmed Rezine and Noomene Ben Henda

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information: Ahmed Rezine <Rezine.Ahemd@it.uu.se> and Noomene Ben Henda <Noomene.BenHenda@it.uu.se>
*/

/**@file tword.h
 *ADT for list-based words in general (parameterized by the type of the symbols).
 *Besides it implements order and upward closed sets of words with respect to that
 *order.
 *
 *@author Ahmed Rezine and Noomene Ben-Henda
 */

#ifndef TWORDOP_H
#define TWORDOP_H

#include<iostream>

#include "tword.h"

/**
 * This class implements the automatic generation of all possible words 
 * resulting from the insertion of an element in the original word and 
 * within a given range.
 */
template<class T>
class TGenerator
{
 private:
  typedef typename TWord<T>::const_iterator iter;
  
 public:
  typedef TWord<T> value_type;

 public:
  //!initialization
  TGenerator() {}

  /** 
   * Insertion within a given range.
   */
    class IWR
    {
    private:
      typedef typename TWord<T>::const_iterator iter;
      
    public:
      //!initialization
      IWR() {}
      IWR(const T& _x, const iter _from , const iter _to , bool e): from(_from), to(_to), cur(_from), x(_x), atEnd(e) {}
	
	//!generate the next possibility
	IWR& operator++()
	  {
	    if (atEnd) {return *this;}
	    if (cur==to) {atEnd=true;}
	    else {++cur;}
	    return *this;
	  }
	
	//!copy
	IWR& operator=(const typename TGenerator<T>::IWR& other)
	  {
	    from=other.from;
	    to=other.to;cur=other.cur;
	    x=other.x;
	    atEnd=other.atEnd;
	    return *this;
	  }

	//end check
	bool end() const
	{
	  return atEnd;
	}
	
	value_type operator*() const
	{
	  value_type temp;
	  temp.insert(temp.end(),from,cur);
	  temp.insert(temp.end(),x);
	  temp.insert(temp.end(),cur,to);
	  return temp;
	}
	
    protected:
	const TGenerator<T>* g;
	iter from;
	iter to;
	iter cur;
	T x;
	bool atEnd;
    };

 public:
    
    //initialized with a given element and a valid range
    IWR start(const T& x, const iter from , const iter to) const 
    { 
      return IWR(x, from, to, false); 
    }
    
 private:
};

#endif



