/*
    Copyright (C) 2005 Ahmed Rezine and Noomene Ben Henda

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information: Ahmed Rezine <Rezine.Ahemd@it.uu.se> and Noomene Ben Henda <Noomene.BenHenda@it.uu.se>
*/

/**@file utilities.h
 *Implementation of various needed methodes to handle predicates and quantifiers.
 *
 *@author Ahmed Rezine and Noomene Ben-Henda
 */

#ifndef UTILITIES_H
#define UTILITIES_H

#include "ref.h"
#include "process.h"

#include <vector>
#include <utility>

//#define UTI_DEBUG
#ifdef UTI_DEBUG
#include <iostream>
#endif

/**
 *Sum class used for computing the "union".
 */
template<class T>
class Sum
{
 public:
  typedef typename T::iterator_type::ref_type ref;
  typedef typename T::iterator_type::value_type value;
  typedef typename std::pair<ref,value> pair_type;
  typedef typename std::vector<pair_type> vec_type; 

 public:
  Sum() {}
  
  template<class Seq>
    bool build_sum_table(vec_type& vec, const T* u_f, const T* e_f, const Seq first, const Seq last, const Seq curr) const
    {
      vec.clear();
      Seq it=first; int i=0;

      bool check=false;
      
      while(it!=last)
	{
	  ref temp;
	  if (it==curr) 
	    {
	      temp=ref(&(e_f->begin(*it)));
	    }
	  else 
	    {
	      temp=ref(&(u_f->begin(*it)));
	      if (!(*temp).end()) {check=true;}
	    }
	  vec.push_back(pair_type(temp,*it));
	  ++i;
	  ++it;
	}
      
      return check;
    }

  bool update(vec_type& vec,int j,bool c)
      {
	if (j==0)
	  {if ((*(vec[0].first)).end()){return false;}
	    else 
	      {if (c) {return true;}
		else { ++(*(vec[0].first));return !(*(vec[0].first)).end();}
	      }
	  }	    
	else
	  {if ((*(vec[j].first)).end())
	      {return update(vec,j-1,c);}
	    else 
	      {if (!c)
		  {++(*(vec[j].first));
		    if ((*(vec[j].first)).end()){return update(vec,j-1,true);}
		    else { return true;}
		  }
		else {return true;}
	      }
	  }
      }
};

/**
 *Product class used for computing the product or "intersection".
 */
template<class T>
class Product
{
 public:
  typedef typename T::iterator_type::ref_type ref;
  typedef typename T::iterator_type::value_type value;
  typedef typename std::pair<const T*,ref> pair_type1;
  typedef typename std::pair<pair_type1,value> pair_type2;
  typedef typename std::vector<pair_type2> vec_type;
  
 public:
  Product() {}
  
  template<class Seq>
    bool build_pro_table(vec_type& vec, const T* u_f, const T* e_f, const Seq first, const Seq last, const Seq curr) const
    {
      vec.clear();
      Seq it=first; int i=0;

      bool check=true;
      
      while(it!=last)
	{
	  pair_type1 temp;
	  if (it==curr) 
	    {
	      temp.first=e_f;
	      temp.second=ref(&(e_f->begin(*it)));
	    }
	  else 
	    {
	      temp.first=u_f;
	      temp.second=ref(&(u_f->begin(*it)));
	      if ((*temp.second).end()) {check=false;}
	    }
	  vec.push_back(pair_type2(temp,*it));
	  ++i;
	  ++it;
	}
      
      return check;
    }
  
  bool update(vec_type& vec, int i) const
  {
    if (i==0)
      {
	if ((*(vec[0].first.second)).end()){return false;}
	else {++(*(vec[0].first.second));return !(*(vec[0].first.second)).end();}
      }	    
    else
      {
	++(*(vec[i].first.second));
	if ((*(vec[i].first.second)).end())
	  {
	    vec[i].first.second=ref(&(vec[i].first.first->begin(vec[i].second)));
	    return update(vec,i-1);
	  }
	else {return true;}
      }
  }
  
};

/**
 *Functions used for composing the constraint generator for guards and the reverse for actions.
 */
template<class T>
class Compose
{
 public:
  typedef typename T::iterator_type::ref_type ref;
  typedef typename T::iterator_type::value_type value;
  typedef typename std::pair<const T*,Ref<value> > pair_type1;
  typedef typename std::pair<ref,pair_type1 > pair_type2;
  typedef typename std::vector<pair_type2> vec_type;
  
 public:
  Compose() {} 
  bool update_down(vec_type& vec,unsigned int i) const;
  bool update_up(vec_type& vec,unsigned int i) const;
  
  template<class Seq>
    bool build_comp_table(vec_type& vec, const Seq from, const Seq to, const process::PWord* w, const process::PWord::const_iterator& pos, const process::Global_Symbol* gs) const
    {
#ifdef UTI_DEBUG
      std::cout << "building composition table" << std::endl;
#endif
      vec.clear();
      Seq iter=from;
      for(;iter!=to;++iter)
	{
#ifdef UTI_DEBUG	  
	  std::cout << "composable function: " << **iter << std::endl;
#endif
	  ref temp_r;
	  if (vec.empty())
	    {
#ifdef UTI_DEBUG
	      std::cout << "given the word: " << *w << ", the position: " << *pos << " and the global symbol: " << *gs << std::endl; 
	      std::cout << "initializing the iterator to : ";
#endif
	      temp_r=ref(&((**iter).begin(w,pos,gs)));
#ifdef UTI_DEBUG
	      std::cout << (**temp_r).word.word << std::endl;
#endif
	    }
	  else 
	    { 
	      temp_r=ref(&(**iter).begin(&vec[0].second.second->word.word,vec[0].second.second->word.curr,&vec[0].second.second->gs));
	    }
	  if ((*temp_r).end()) {return false;}
	  pair_type1 temp_p1(&(**iter),Ref<value>(new value(**temp_r)));
	  pair_type2 temp_p2(temp_r,temp_p1);
	  vec.insert(vec.begin(),temp_p2);
#ifdef UTI_DEBUG
	  std::cout << "inserting: " << *(temp_p2.second.first) << " curr word: " << temp_p2.second.second->word.word << std::endl;
#endif
	}
#ifdef UTI_DEBUG
      std::cout << "building composition table done" << std::endl;
#endif
      return true;
    }

};

template<class T>
bool Compose<T>::update_up(vec_type& vec,unsigned int i) const
{
#ifdef UTI_DEBUG
  std::cout << "updating up composition table " << i << std::endl;
#endif
  if (vec.empty() || i >= vec.size()) {return false;}

#ifdef UTI_DEBUG
  std::cout << "incrementing the iterator of " <<  *(vec[i].second.first) << std::endl;
  std::cout << "form :" << (*(*vec[i].first)).word.word << std::endl;
#endif

  ++(*(vec[i].first));

#ifdef UTI_DEBUG
  std::cout << "to : " << (*(*vec[i].first)).word.word << std::endl;
#endif
  
  if ((*(vec[i].first)).end())
    {
      if (i==vec.size()-1) 
	{
#ifdef UTI_DEBUG
	  std::cout << "updating up reached the end of the top " << std::endl;
#endif  
	  return false;
	}
      else 
	{return update_up(vec,i+1);}
    }
  else 
    {
      vec[i].second.second=Ref<value>(new value(**vec[i].first));
#ifdef UTI_DEBUG
      std::cout << "incrementing the current iterator to: " << (**vec[i].first).word.word << std::endl;
#endif       
      if (i>0) {return update_down(vec,i-1);}
    }
#ifdef UTI_DEBUG 
  std::cout << "updating up finished successefully" << std::endl;
#endif
  return true;
}

template<class T>
bool Compose<T>::update_down(vec_type& vec,unsigned int i) const
{
#ifdef UTI_DEBUG 
  std::cout << "updating down composition table" << i << std::endl;
#endif
  if ((i>=0)) 
    {
      vec[i].first=ref(&vec[i].second.first->begin(&vec[i+1].second.second->word.word,vec[i+1].second.second->word.curr,&vec[i+1].second.second->gs));
      if ((*vec[i].first).end()) 
	{
#ifdef UTI_DEBUG 
	  std::cout << "updating down stopped because failed to initialize iterator" << std::endl;
#endif	  
	  return update_up(vec,i+1);
	}
      else
	{
	  vec[i].second.second=Ref<value>(new value(**vec[i].first));
	  if (i>0) {return update_down(vec,i-1);}
	}
    }
#ifdef UTI_DEBUG 
  std::cout << "updating down finished successefully" << std::endl;
#endif
  return true;
}

#endif

